#
# Author: Shuwen Deng <shuwen.deng@yale.edu>
# Date:   08.27.2019
#
# This file converts new instruction to binary
#
# This code is released under same license as RocketChip code,
# license details can be found in LICENSE.SiFive file.
#


""" convert new instruction to binary"""
import sys
import os

def convert_reg(in_value):
    options ={"x0": "00000",
	      "x1": "00001",
	      "x2": "00010",
	      "x3": "00011",
	      "x4": "00100",
	      "x5": "00101",
	      "x6": "00110",
	      "x7": "00111",
	      "x8": "01000",
	      "x9": "01001",
	      "x10": "01010",
	      "x11": "01011",
	      "x12": "01100",
	      "x13": "01101",
	      "x14": "01110",
	      "x15": "01111",
	      "x16": "10000",
	      "x17": "10001",
	      "x18": "10010",
	      "x19": "10011",
	      "x20": "10100",
	      "x21": "10101",
	      "x22": "10110",
	      "x23": "10111",
	      "x24": "11000",
	      "x25": "11001",
	      "x26": "11010",
	      "x27": "11011",
	      "x28": "11100",
	      "x29": "11101",
	      "x30": "11110",
	      "x31": "11111",
    }
    tmp = options[in_value]
    return tmp

def convert_funct3(in_value):
    options = {"sdrand": "000",
           "sdnorm": "001",
	       "ldrand": "010",
           "ldnorm": "011"
    }
    tmp = options[in_value]
    return tmp

def convert_imm(in_value):
    tmp_int = int(in_value)
    tmp =str('{0:012b}'.format(tmp_int & 0xfff))

    return tmp


def convert(line, inst_name):
    """ convert input line to binary according to the format"""
    binary_inst= "" #put your encoder here
    # pure added new instructions, not suit for imporved tests
    line_list = line.split(" ")
    inst_index = line_list.index(inst_name)
    # add pre part
    for x in range(0, inst_index):
	binary_inst = binary_inst + line_list[x] + " "
    inst = line_list[inst_index]
    # check the consistency of inst name
    if inst != inst_name:
	print "Inconsistent inst name " + inst_name + " and " + inst
    rd_rs2 = line_list[inst_index+1]
    rs1 = line_list[inst_index+2]
    real_rd_rs2 = convert_reg(rd_rs2[:-1]);
    imm = convert_imm(rs1.split('(')[0]);
    tmp_rs1 = (rs1.split('(')[1]).split(')')[0]
    real_rs1 = convert_reg(tmp_rs1);
    opcode = "1101011"
    final_binary_inst = ""
    # put binary numbers to their own position
    if(inst=="sdrand"): 
        final_binary_inst = " .word 0b" + imm[0:7] + real_rd_rs2 + real_rs1 + convert_funct3(inst) + imm[7:12] + opcode + " "
    elif(inst=="ldrand"): 
        final_binary_inst = " .word 0b" + imm + real_rs1 + convert_funct3(inst) + real_rd_rs2 + opcode + " "
    elif(inst=="sdnorm"): 
        final_binary_inst = " .word 0b" + imm[0:7] + real_rd_rs2 + real_rs1 + convert_funct3(inst) + imm[7:12] + opcode + " "
    elif(inst=="ldnorm"): 
        final_binary_inst = " .word 0b" + imm + real_rs1 + convert_funct3(inst) + real_rd_rs2 + opcode + " "
    else:
        print "Invalid instruction format " + inst
    # add post part
    binary_inst = binary_inst + final_binary_inst
    for x in range(inst_index+3, len(line_list)):
	binary_inst = binary_inst + line_list[x] + " "
    return (binary_inst+"\n")

root_dir =  r""+(str(sys.argv[1]))+"/ass_pre_tests"
output_dir =  r""+(str(sys.argv[1]))+"/ass_s_tests"


for file in os.listdir(root_dir):
    filename = root_dir + "/" + file
    
    in_filename = filename #+ ".Spre"
    out_filename = output_dir + "/" + file[0:-3] # + ".S"

    F_in = open(in_filename, "rb")
    F_out= open(out_filename, "wb")

    """ convert and write to output file """
    for line in F_in:
           if line.split(" ").__contains__("ldrand"):
            line=convert(line, "ldrand")
           elif line.split(" ").__contains__("sdrand"):
            line=convert(line, "sdrand")
           elif line.split(" ").__contains__("ldnorm"):
            line=convert(line, "ldnorm")
           elif line.split(" ").__contains__("sdnorm"):
            line=convert(line, "sdnorm")
           F_out.write(line)
            
    F_in.close()
    F_out.close()

