// See LICENSE.Berkeley for license details.
// See LICENSE.SiFive for license details.
//
// Author: Shuwen Deng <shuwen.deng@yale.edu>
// Date:   08.27.2019
//
// This file edits the event related registers for secure TLB project
//
// This code is released under same license as RocketChip code,
// license details can be found in LICENSE.SiFive file.
//

package freechips.rocketchip.rocket

import Chisel._
import freechips.rocketchip.util._
import freechips.rocketchip.util.property._

class EventSet(gate: (UInt, UInt) => Bool, events: Seq[(String, () => Bool)]) {
  def size = events.size
  def hits = events.map(_._2()).asUInt
  //SD: try solve the bit problem
  def inter = events.map(_._2())
  def check(mask: UInt):Bool = {
    val return_val = Reg(false.B)
    when (mask < size.U){
      return_val := inter(mask)
    } 
    return_val
  }
  def dump() {
    for (((name, _), i) <- events.zipWithIndex)
      when (check(1.U << i)) { printf(s"Event $name\n") }
  }
  // SD: for testing
  def name = events.map(_._1)
  def event_element = events

  def withCovers {
    events.zipWithIndex.foreach {
      case ((name, _), i) => cover(check(1.U << i), name)
    }
  }
}

/*class EventSet(gate: (UInt, UInt) => Bool, events: Seq[(String, () => Bool)]) {
  def size = events.size
  def hits = events.map(_._2()).asUInt
  def check(mask: UInt) = gate(mask, hits)
  def dump() {
    for (((name, _), i) <- events.zipWithIndex)
      when (check(1.U << i)) { printf(s"Event $name\n") }
  }
  def withCovers {
    events.zipWithIndex.foreach {
      case ((name, _), i) => cover(check(1.U << i), name)
    }
  }
}*/

class EventSets(val eventSets: Seq[EventSet]) {
  def maskEventSelector(eventSel: UInt): UInt = {
    // allow full associativity between counters and event sets (for now?)
    val setMask = (BigInt(1) << log2Ceil(eventSets.size)) - 1
    val maskMask = ((BigInt(1) << eventSets.map(_.size).max) - 1) << eventSetIdBits
    eventSel & (setMask | maskMask).U
  }

  private def decode(counter: UInt): (UInt, UInt) = {
    require(eventSets.size <= (1 << eventSetIdBits))
    // SD: modify mask to enable the counter
    val set = Reg(init = UInt(0,64))
    val mask = Reg(init = UInt(0,64))
    val flag = Reg(init = UInt(0,64))
    val base = Reg(init = UInt(0,64))
    val counter_reg = Reg(init = counter)
    val set_reg = Reg(init = UInt(0,64))
    eventSets foreach (x => (
      //base := UInt(x.size) + base
      //println("x.size is now" + x.size)
      when(flag===0.U && (counter_reg < UInt(x.size))){
        mask:=counter_reg;
        set:=set_reg;
        flag:=UInt(1)
      }
    ))
    (set, mask)
  }

  /*private def decode(counter: UInt): (UInt, UInt) = {
    require(eventSets.size <= (1 << eventSetIdBits))
    (counter(log2Ceil(eventSets.size)-1, 0), counter >> eventSetIdBits)
  }*/

  def evaluate(eventSel: UInt): Bool = {
    val (set, mask) = decode(eventSel)
    val sets = eventSets map (_ check mask)
    sets(set)
  }

  def cover() = eventSets.foreach { _ withCovers }

  private def eventSetIdBits = 8
}
